import os,re,shutil
from kcw import config as conf
from kcw.common.autoload import *
class Exec:
    runtimepath='runtime/app'
    sourcepath=''
    appname="application" #应用名
    name='home' #应用模块名字
    config=conf
    def init(self,config=None):
        if config:
            self.config=config
        path=self.runtimepath+'/'+self.appname+'/'+self.name
        plists=['view'] #模块下要执行的文件夹
        for sourcep in plists:
            if not os.path.exists(path+'/'+sourcep):
                os.makedirs(path+"/"+sourcep)
            if sourcep=='view':
                self.__copyview(sourcep)
            elif sourcep=='config' or sourcep=='common':
                self.__copyweb(sourcep)
    def __copyweb(self,sourcep):
        "处理web模块"
        path1=self.sourcepath+"/"+sourcep
        path2=self.runtimepath+'/'+self.appname+'/'+self.name+"/"+sourcep
        # print(path1,path2)
        if os.path.exists(path1):
            lists=os.listdir(path1)
            for files in lists:
                # print(files)
                if os.path.isfile(path1+"/"+files):
                    f=open(path1+"/"+files,"r",encoding='utf-8')
                    filecon=f.read()
                    f.close()
                    f=open(path2+"/"+files,"w+",encoding='utf-8')
                    f.write(filecon)
                    f.close()
                elif files!='__pycache__':
                    if not os.path.exists(path2+"/"+files):
                        os.makedirs(path2+"/"+files)
                    self.__copyweb(sourcep+"/"+files)
    def __copyview(self,sourcep):
        """处理视图文件"""
        path1=self.sourcepath+"/"+sourcep
        path2=self.runtimepath+'/'+self.appname+'/'+self.name+"/"+sourcep
        lists=os.listdir(path1)
        route=self.config.route
        wjjs=sourcep.split("/")
        qmly=''
        i=0
        while i < len(wjjs):
            if i>0:
                qmly=qmly+"/"+wjjs[i]
            i=i+1
        for files in lists:
            if os.path.isfile(path1+"/"+files):
                if '.py' in files:  #只操作包含 .py 的文件
                    filecon=''
                    f=open(path1+"/"+files,"r",encoding='utf-8')
                    fileind=re.sub('.py','',files)
                    if files == '__init__.py':
                        filecon=""
                        # print(files)
                        while True:
                            line = f.readline()
                            if not line:
                                break
                            if 'Blueprint(' in line:
                                # print(files)
                                line=(self.name+"app = Blueprint(\n"+
                                    "   'application',\n"+
                                    "   __name__,\n"+
                                    "   static_folder='../../../../../app/"+self.name+"/'+app['static_folder'],#设置静态url目录\n"+
                                    "   template_folder='../../../../../app/"+self.name+"/'+app['template_folder'],#设置模板文件目录\n"+
                                    ")")
                                filecon=filecon+line+"\n"
                            else:
                                filecon=filecon+line
                    else:
                        ddddddd=''
                        for k in wjjs:
                            ddddddd=ddddddd+"."
                        filecon='from '+ddddddd+' import '+self.name+'app\n'
                        while True:
                            line = f.readline()
                            # if(files=='__init__.py'):
                            #     print(line)
                            if not line:
                                break
                            if 'def ' in line:
                                strs=re.sub('def ','',line)
                                fun=strs[0:strs.rfind('(')]
                                fun=re.sub(' ','',fun)
                                ly=''
                                routebs=qmly+"/"+fileind+"/"+fun
                                cs=re.findall(re.compile(r'[(](.*?)[)]', re.S), line)[0]
                                if routebs in route:# and 'route' in route[routebs]:# and route[routebs]['route']:  #添加配置里的路由
                                    if isinstance(route[routebs],dict):
                                        if 'methods' in route[routebs]:
                                            methods=str(route[routebs]['methods'])
                                        else:
                                            methods=str(route['methods'])
                                        ly=ly+'@'+self.name+'app.route("'+route[routebs]['route']+'",methods='+methods+',strict_slashes=False)\n'
                                        # print(cs)
                                        if '<' in route[routebs]['route'] and '>' in route[routebs]['route']:
                                            pass
                                        else:
                                            if cs:
                                                cs=cs.split(",")
                                                s=''
                                                for kkk in cs:
                                                    kkk=kkk.split("=")
                                                    s=s+kkk[0]+'/<'+kkk[0]+'>'
                                                    ly=ly+'@'+self.name+'app.route("'+route[routebs]['route']+s+'",methods='+methods+',strict_slashes=False)\n'
                                    elif isinstance(route[routebs],list):
                                        for v in route[routebs]:
                                            if 'methods' in v:
                                                methods=str(v['methods'])
                                            else:
                                                methods=str(route['methods'])
                                            ly=ly+'@'+self.name+'app.route("'+v['route']+'",methods='+methods+',strict_slashes=False)\n'
                                            # print("*"*5)
                                            if '<' in v['route'] and '>' in v['route']:
                                                pass
                                            else:
                                                if cs:
                                                    cs=cs.split(",")
                                                    s=''
                                                    for kkk in cs:
                                                        kkk=kkk.split("=")
                                                        s=s+kkk[0]+'/<'+kkk[0]+'>'
                                                        ly=ly+'@'+self.name+'app.route("'+v['route']+s+'",methods='+methods+',strict_slashes=False)\n'
                                elif route['default']:  #添加默认路由
                                    methods=str(route['methods'])
                                    if fun =='index':
                                        ly=ly+'@'+self.name+'app.route("'+qmly+"/"+fileind+'",methods='+methods+',strict_slashes=False)\n'
                                        if fileind == 'index':
                                            if not qmly:
                                                ly=ly+'@'+self.name+'app.route("/",methods=["POST","GET"],strict_slashes=False)\n'
                                            else:
                                                ly=ly+'@'+self.name+'app.route("'+qmly+'",methods=["POST","GET"],strict_slashes=False)\n'
                                    ly=ly+'@'+self.name+'app.route("'+qmly+"/"+fileind+'/'+fun+'",methods='+methods+',strict_slashes=False)\n'
                                    if cs:
                                        cs=cs.split(",")
                                        s=''
                                        for kkk in cs:
                                            kkk=kkk.split("=")
                                            if route['defaultparam']==1:
                                                s=s+'/'+kkk[0]+'/<'+kkk[0]+'>'
                                            else:
                                                s=s+'/<'+kkk[0]+'>'
                                            # print(fun+s)
                                            ly=ly+'@'+self.name+'app.route("'+"/"+fileind+'/'+fun+s+'",methods='+methods+',strict_slashes=False)\n'
                                line=re.sub('def ','',line)
                                qmlys=re.sub('/','_',qmly)
                                filecon=filecon+ly+'def kcw_'+self.appname+"_"+self.name+qmlys+"_"+fileind+"_"+line
                            else:
                                filecon=filecon+line
                    f.close()
                    f=open(path2+"/"+files,"w+",encoding='utf-8')
                    f.write(filecon)
                    f.close()
            elif files != '__pycache__':
                if not os.path.exists(path2+"/"+files):
                    os.makedirs(path2+"/"+files)
                self.__copyview(sourcep+"/"+files)


# 应用配置
app={}
app['app_name']='' #应用名称
app['app_debug']=True  #是否开启调试模式
app['SECRET_KEY']='reresgrdsafagrewgregrtdge' #session密钥
app['template_folder']='temp'  #设置模板文件目录名 注意：不能配置目录路径 
app['static_folder']='static/' #设置静态url目录名

# 默认数据库配置
database={}
database['type']='mysql' # 数据库类型  目前支持mysql
database['host']=['127.0.0.1']#服务器地址 [地址1,地址2,地址3...] 多个地址分布式(主从服务器)下有效
database['port']=[3306] #端口 [端口1,端口2,端口3...]
database['user']=['root']  #用户名 [用户名1,用户名2,用户名3...]
database['password']=['root']  #密码 [密码1,密码2,密码3...]
database['db']=['test']  #数据库名 [数据库名1,数据库名2,数据库名3...]
database['charset']='utf8'   #数据库编码默认采用utf8
database['pattern']=False # True数据库长连接模式 False数据库短连接模式  注：建议web应用使用短连接，cli应用使用长连接
database['cli']=False # 是否以cli方式运行
database['dbObjcount']=1 # 连接池数量（单个数据库地址链接数量），数据库链接实例数量 mysql长链接模式下有效
database['deploy']=0 # 数据库部署方式:0 集中式(单一服务器),1 分布式(主从服务器)  mysql数据库有效
database['master_num']=1 #主服务器数量 不能超过host服务器数量  （等于服务器数量表示读写不分离：主主复制。  小于服务器表示读写分离：主从复制。） mysql数据库有效
database['master_dql']=False #主服务器是否可以执行dql语句 是否可以执行select语句  主服务器数量大于等于host服务器数量时必须设置True
database['break']=0 #断线重连次数，0表示不重连。 注：cli模式下 10秒进行一次重连并且连接次数是当前配置的300倍
# redis配置
redis={}
redis['host']='127.0.0.1' #服务器地址
redis['port']=6379 #端口
redis['password']='fk459915476'  #密码
redis['db']=0 #Redis数据库    注：Redis用0或1或2等表示
redis['pattern']=True # True连接池链接 False非连接池链接

#缓存配置
cache={}
cache['type']='File' #驱动方式 支持 File Redis
cache['path']='cachepath/' #缓存保存目录 
cache['expire']=120 #缓存有效期 0表示永久缓存
cache['host']=redis['host'] #Redis服务器地址
cache['port']=redis['port'] #Redis 端口
cache['password']=redis['password'] #Redis登录密码
cache['db']=1 #Redis数据库    注：Redis用1或2或3等表示

#路由配置
route={}
route['default']=True #是否开启默认路由  默认路由开启后面不影响以下配置的路由，文件名和方法名作为路由地址   如：index.py的index方法的默认路由为: /index/index  或/index/index/   #   以下是路由配置
route['defaultparam']=1 #默认路由参数，1表示路由参数中需要参数名   0表示不需要
route['methods']=["POST","GET"] #默认请求方式
# route['/v1/index/test1']={'route':"/test","methods":["GET"]}  #设置view/v1/index.py的test1方法的路由为: /test  并且设置请求方式为GET  配置路由后，该路由的默认路由将不会生效
# route['/v1/login/login']={'route':"/login"}  #设置view/v1/login.py的login方法的路由为: /login 请求方式使用默认


oauth={}
oauth['status']=False   #签权开关
oauth['need']=[	        # “需要” 签权的视图函数
	"*" 	            # *：表示全部。 /index*:表示index视图文件下的所有方法。  /v1*：表示v1下的所有。/v1/index*：表示v1/index下的所有
]
oauth['unwanted']=[
	"login/*"	        #表示login下所有不需要签权
]
#其他配置
other={}


confkcw={}
confkcw['version']='1.4.0'
confkcw['name']='kcw'
confkcw['description']='kcw 一个快速搭建cli和web的框架,修复Http类请求头以及Http set_cookie字符串转字典'
confkcw['author_email']='fk1402936534@qq.com'
confkcw['install_requires']=['pymongo==3.9.0','six==1.12.0','requests==2.22.0']


import os,re,traceback,shutil,platform
# import config
from kcw.common.autoload import *
from kcw.config import *
from .exec import Exec
class create:
    name="home"  #应用模块名字
    appname="application"
    runtimepath='runtime/app'  #应用生成目录
    host='0.0.0.0' #绑定ip
    port='39001' #绑定端口
    path=os.path.split(os.path.realpath(__file__))[0] #当前文件目录
    framepath="."#框架目录
    def init(self):
        # if 'Windows' in platform.system():
        #     # os.popen('setx PATH "%PATH%;'+get_folder()+'/kcw'+str(confkcw['version'])+'/app/binaryfile"  /M')
        #     os.popen('setx PATH "%PATH%;'+get_folder()+'/kcw/app/binaryfile"  /M')
        # if 'Linux' in platform.system():
        #     try:
        #         PATH=os.environ['PATH']
        #         PATH=PATH.split(":")[-1]
        #     except:
        #         PATH=None
        #     if PATH:  #设置软链接
        #         os.system("ln -s "+get_folder()+"/kcw/binaryfile/kcw /usr/bin")
        if not os.path.exists(self.framepath+"/common"):
            os.makedirs(self.framepath+"/common")
        if not os.path.isfile(self.framepath+"/common/__init__.py"):
            f=open(self.framepath+"/common/__init__.py","a",encoding='utf-8')
            f.write("from kcw.common import *")
            f.close()
        if not os.path.exists(self.framepath+"/config"):
            os.makedirs(self.framepath+"/config")
        if not os.path.isfile(self.framepath+"/config/__init__.py"):
            f=open(self.framepath+"/config/__init__.py","a",encoding='utf-8')
            f.write("from kcw.config import *")
            f.close()
        # if not os.path.isfile(self.framepath+"/config/__init__.py"):
        #     f=open(self.framepath+"/config/__init__.py","a",encoding='utf-8')
        #     f.write("from .conf import *")
        #     f.close()
    def cliapp(self):
        self.init()
        "创建cli应用"
        framepath=self.framepath+"/"+self.appname+"/"+self.name+"/"
        if os.path.exists(framepath):
            print(framepath,"cli应用和模块已存在，已跳过")
            # exit()
        else:
            shutil.copytree(self.path+"/app/cli",self.appname+'/'+self.name)
            self.__execcommon(self.appname+'/'+self.name+"/common/__init__.py")
            self.__cli_exec_start(self.appname+'/'+self.name+"/start/start.py")
    def __cli_exec_start(self,pathfile):
        "处理cli下start.py文件"
        filecon=''
        f=open(pathfile,"r",encoding='utf-8')
        filecon='# -*- coding: utf-8 -*-\nimport sys\nsys.path.append("../../../")\nfrom '+self.appname+'.'+self.name+'.common import *\n'
        while True:
            line = f.readline()
            if not line:
                break
            filecon=filecon+line
        f.close()
        f=open(pathfile,"w+",encoding='utf-8')
        f.write(filecon)
        f.close()

        
    def webapp(self):
        """创建web应用 """
        self.init()
        framepath=self.framepath+"/"+self.appname+"/"+self.name+"/"
        if os.path.exists(framepath):
            print(framepath,"web应用和模块已存在，已跳过")
            # exit()
        else:
            shutil.copytree(self.path+"/app/web", "./"+self.appname+'/'+self.name)
            self.__execview(self.appname+'/'+self.name+"/view")
            self.__execcommon(self.appname+'/'+self.name+"/common/__init__.py")
            if not os.path.exists(framepath+"/start"):
                os.makedirs(framepath+"/start")
            f=open(framepath+"/start/server.py","w+",encoding='utf-8')
            f.write('import sys\n'+
            'sys.path.append("../../../")\n'+
            'from '+self.appname+'.'+self.name+'.common import *\n'+
            'if config.app["app_debug"]:\n'+
            '    from kcw.exec import Exec\n'+
            '    e=Exec()\n'+
            '    e.runtimepath="../../../'+self.runtimepath+'"\n'+
            '    e.sourcepath="../"\n'+
            '    e.appname="'+self.appname+'"\n'+
            '    e.name="'+self.name+'"\n'+
            '    e.init(config)\n'+
            '    from '+self.appname+'.'+self.name+'.view import '+self.name+'app\n'+
            'from kcw.packages.flask import Flask\n'+
            'from runtime.app.'+self.appname+'.'+self.name+'.view import '+self.name+'app\n'+
            'flaskapp =Flask(__name__)\n'+
            'flaskapp.register_blueprint('+self.name+'app)\n'+
            'class flaskconfig():\n'+
            '    DEBUG=config.app["app_debug"]\n'+
            '    SECRET_KEY=config.app["SECRET_KEY"]\n'+
            'flaskapp.config.from_object(flaskconfig)\n'+
            '@flaskapp.errorhandler(404)\n'+
            'def err404(err):\n'+
            '    return returnjson([],404,"访问的URL地址错误",404)\n'+
            '@flaskapp.errorhandler(403)\n'+
            'def err403(err):\n'+
            '    return returnjson([],403,"资源不可用",403)\n'+
            '@flaskapp.errorhandler(500)\n'+
            'def err500(err):\n'+
            '    return returnjson([],500,"内部服务器错误，请稍后在试一试",500)\n'+
            '@flaskapp.before_request\n'+
            'def handel_before_request():\n'+
            '    "签权验证"\n'+
            '    if app.oauth(request.path,config):\n'+
            '       try:\n'+
            '           oa=oauth(request.path)\n'+
            '       except NameError:\n'+
            '           print("您已开启签权验证，但您却没有定义签权方法，您应该在'+self.appname+'/'+self.name+'/common/autoload.py文件中定义oauth方法并且接收一个参数")\n'+
            '       else:\n'+
            '           if not oa[0]:\n'+
            '               return oa[1]\n'+
            '@flaskapp.after_request  #请求后执行\n'+
            'def after_request(response):\n'+
            '    return response\n'+
            'if __name__ == "__main__":\n'+
            '    flaskapp.run(host="'+self.host+'",port="'+self.port+'")\n')
            f.close()
        self.__execruntimepath()
    def __execview(self,path):
        "处理视图文件"
        lists=os.listdir(path)
        for files in lists:
            if os.path.isfile(path+"/"+files):
                filecon=''
                f=open(path+"/"+files,"r",encoding='utf-8')
                if files=='__init__.py':
                    while True:
                        line = f.readline()
                        if not line:
                            break
                        if 'Blueprint(' in line and '=' in line:
                            line=self.name+line
                        filecon=filecon+line
                else:
                    filecon='from '+self.appname+'.'+self.name+'.common import *\n'
                    while True:
                        line = f.readline()
                        if not line:
                            break
                        filecon=filecon+line
                f.close()
                f=open(path+"/"+files,"w+",encoding='utf-8')
                f.write(filecon)
                f.close()
            elif files != '__pycache__':
                if not os.path.exists(path+"/"+files):
                    os.makedirs(path+"/"+files)
                else:
                    self.__execview(path+"/"+files)
    def __execcommon(self,pathfile):
        f=open(pathfile,"r",encoding='utf-8')
        filecon='from common import *\n'+'from '+self.appname+'.'+self.name+' import config\n'
        while True:
            line = f.readline()
            if not line:
                break
            filecon=filecon+line
        f.close()
        f=open(pathfile,"w+",encoding='utf-8')
        f.write(filecon)
        f.close()
    def __execruntimepath(self):
        e=Exec()
        e.runtimepath=self.framepath+"/"+self.runtimepath
        e.sourcepath=self.framepath+"/"+self.appname+"/"+self.name
        e.appname=self.appname
        e.name=self.name
        e.init()
        


from kcw.packages.flask import request,render_template,session
#当前文件下定义的函数只能当前模块下可以使用
PATH_MOD=os.path.dirname(__file__)+"/../" #当前模块目录
def returnjson(data=[],code=0,msg="成功",status=200):
    """在浏览器输出包装过的json

        参数 data 结果 默认[]

        参数 code body状态码 默认0

        参数 msg body状态描述 默认 成功

        参数 status http状态码 默认 200

        返回 json字符串结果集 
        """
    res={
        "code":code,
        "msg":msg,
        "time":times(),
        "data":data
    }
    return json_encode(res),status,{"Content-Type":"application/json; charset=utf-8"}
def template(template_name_or_list, **context):
    return render_template(template_name_or_list, **context)
def oauth(path):
    "权限验证方法,需要您自己完成逻辑"
    if False:
        data=True,returnjson({},0,'验证成功',200)
    else:
        data=False,returnjson({},-1,'签权验证失败',503)
    return data


from kcw.packages import redis as red
import kcw.config as config
class redis:
    "redis  注意：连接池链接模式下不支持动态配置"
    __redisObj=None
    __config=config.redis
    def __connects(self):
        """设置redis链接"""
        if self.__config['pattern']:
            if not self.__redisObj:
                if self.__config['password']:
                    redis_pool=red.ConnectionPool(host=self.__config['host'],password=self.__config['password'],port=self.__config['port'],db=self.__config['db'])
                else:
                    redis_pool=red.ConnectionPool(host=self.__config['host'],port=self.__config['port'],db=self.__config['db'])
                self.__redisObj=red.Redis(connection_pool=redis_pool)
        else:
            if self.__config['password']:
                self.__redisObj=red.Redis(host=self.__config['host'],password=self.__config['password'],port=self.__config['port'],db=self.__config['db'])
            else:
                self.__redisObj=red.Redis(host=self.__config['host'],port=self.__config['port'],db=self.__config['db'])
            # print(self.__redisObj)
    def getconfig(self):
        return self.__config
    def connect(self,config):
        """设置redis链接信息 

        参数 config 参考配置信息格式

        返回 redis
        """ 
        if config:
            if isinstance(config,dict):
                if "host" in config:
                    self.__config['host']=config['host']
                if "port" in config:
                    self.__config['port']=config['port']
                if "password" in config:
                    self.__config['password']=config['password']
                if "db" in config:
                    self.__config['db']=config['db']
        return self
    def redisObj(self):
        "得到一个redis连接对象，执行更多高级操作"
        self.__connects()
        return self.__redisObj
    def set(self,name,value,ex=None, px=None, nx=False, xx=False):
        """
        name，键

        value，值

        ex，过期时间（秒）

        px，过期时间（毫秒）

        nx，如果设置为True，则只有key不存在时，当前set操作才执行,同#setnx(key, value)

        xx，如果设置为True，则只有key存在时，当前set操作才执行
        """
        if not self.__redisObj:
            self.__connects()
        return self.__redisObj.set(name, value, ex=ex, px=px, nx=nx, xx=xx)
    def get(self,name):
        """获取name的值

        name，键
        返回键“name”处的值，如果该键不存在，则返回“none”
        """
        self.__connects()
        return self.__redisObj.get(name)
    def rpush(self,name, *values):
        "元素从list的右边加入"
        self.__connects()
        # print(self.__config)
        return self.__redisObj.rpush(name, *values) 
    def rpop(self,name):
        "元素从list的右边移出"
        self.__connects()
        return self.__redisObj.rpop(name)
    def rpoplpush(self,src, dst):
        "元素从list的右边移出,并且从list的左边加入"
        self.__connects()
        return self.__redisObj.rpoplpush(src, dst)
    def rpushx(self,name, value):
        "当name存在时，元素才能从list的右边加入"
        self.__connects()
        return self.__redisObj.rpushx(name, value)
    def lpush(self,name, *values):
        "元素从list的左边加入，可以添加多个"
        self.__connects()
        return self.__redisObj.lpush(name, *values)
    def lpop(self,name):
        "元素从list的左边移出"
        self.__connects()
        return self.__redisObj.lpop(name)
    def lpushxs(self,name):
        "当name存在时，元素才能从list的左边加入"
        self.__connects()
        return self.__redisObj.lpushx(name)
    def hmset(self,name,mapping):
        "在hash“name”中为每个键设置值"
        self.__connects()
        return self.__redisObj.hmset(name,mapping)
    def hmget(self,name, keys, *args):
        "返回与“keys”顺序相同的值列表``"
        self.__connects()
        return self.__redisObj.hmget(name, keys, *args)



import time,hashlib,json,re,os
import datetime as core_datetime
# import config
from kcw.packages.dateutil.relativedelta import relativedelta as core_relativedelta
from kcw.utill.db import mysql
from kcw.utill.db import mongodb
from kcw.utill.cache import cache
from kcw.utill.redis import redis as rediss
from kcw.utill.http import Http
from kcw.utill import app
redis=rediss()
def M(table=None,config=None):
    """数据库操作实例
    
    参数 table：表名

    参数 config 数据库配置  可以传数据库名字符串
    """
    dbs=mysql.mysql()
    if table is None:
        return dbs
    elif config:
        return dbs.connect(config).table(table)
    else:
        return dbs.table(table)
def mongo(table=None,config=None):
    """mongodb数据库操作实例
    
    参数 table：表名(mongodb数据库集合名)

    参数 config mongodb数据库配置  可以传数据库名字符串
    """
    mObj=mongodb.mongo()
    if table is None:
        return mObj
    elif config:
        return mObj.connect(config).table(table)
    else:
        return mObj.table(table)
def is_index(params,index):
    """判断列表或字典里的索引是否存在
    params  列表或字典
    index   索引值
    return True/False
    """
    try:
        params[index]
    except KeyError:
        return False
    except IndexError:
        return False
    else:
        return True
def set_cache(name,values,expire="no"):
    """设置缓存

        参数 name：缓存名

        参数 values：缓存值

        参数 expire：缓存有效期 0表示永久  单位 秒
        
        return Boolean类型
        """
    return cache.cache().set_cache(name,values,expire)
def get_cache(name):
    """获取缓存

    return 或者的值
    """
    return cache.cache().get_cache(name)
def del_cache(name):
    """删除缓存

    return Boolean类型
    """
    return cache.cache().del_cache(name)
def md5(strs):
    """md5加密"""
    m = hashlib.md5()
    b = strs.encode(encoding='utf-8')
    m.update(b)
    return m.hexdigest()
def times():
    """时间戳 精确到秒"""
    return int(time.time())
# def json_decode(jsonstr):
#     """json字符串转python类型"""
#     try:
#         return eval(jsonstr)
#     except Exception:
#         return {}
def json_decode(strs):
    """json字符串转python类型"""
    try:
        return json.loads(strs)
    except Exception:
        return {}
def json_encode(strs):
    """转成字符串"""
    try:
        return json.dumps(strs,ensure_ascii=False)
    except Exception:
        return {}
def dateoperator(date,years=0,formats='%Y%m%d%H%M%S',months=0, days=0, hours=0, minutes=0,seconds=0,
                 leapdays=0, weeks=0, microseconds=0,
                 year=None, month=None, day=None, weekday=None,
                 yearday=None, nlyearday=None,
                 hour=None, minute=None, second=None, microsecond=None):
    """日期相加减计算
    date 2019-10-10
    formats 设置需要返回的时间格式 默认%Y%m%d%H%M%S
    
    years 大于0表示加年  反之减年
    months 大于0表示加月  反之减月
    days 大于0表示加日  反之减日

    return %Y%m%d%H%M%S
    """
    formatss='%Y%m%d%H%M%S'
    date=re.sub('[-年/月:：日 时分秒]','',date)
    if len(date) < 8:
        return None
    if len(date) < 14:
        s=14-len(date)
        i=0
        while i < s:
            date=date+"0"
            i=i+1
    d = core_datetime.datetime.strptime(date, formatss)
    strs=(d + core_relativedelta(years=years,months=months, days=days, hours=hours, minutes=minutes,seconds=seconds,
                 leapdays=leapdays, weeks=weeks, microseconds=microseconds,
                 year=year, month=month, day=day, weekday=weekday,
                 yearday=yearday, nlyearday=nlyearday,
                 hour=hour, minute=minute, second=second, microsecond=microsecond))
    # strs=re.sub('[-: ]','',str(strs))
    strs=strs.strftime(formats)
    return strs
def get_folder():
    '获取当前框架目录'
    path=os.path.split(os.path.realpath(__file__))[0] #当前文件目录
    framepath=path.split('\\') ##框架主目录
    s=''
    for k in framepath:
        s=s+'/'+k
    framepath=s[1:]
    return re.sub('/kcw/common','',framepath) #包所在目录
aa=[]
def get_file(folder='./',is_folder=True,suffix="*",lists=[]):
    """获取文件夹下所有文件夹和文件

    folder 要获取的文件夹路径

    is_folder  是否返回列表中包含文件夹

    suffix 获取指定后缀名的文件 默认全部
    """
    lis=os.listdir(folder)
    for files in lis:
        if not os.path.isfile(folder+"/"+files):
            if is_folder:
                zd={"type":"folder","path":folder+"/"+files}
                lists.append(zd)
            get_file(folder+"/"+files,is_folder,suffix)
        else:
            if suffix=='*':
                zd={"type":"file","path":folder+"/"+files}
                lists.append(zd)
            else:
                if files[-(len(suffix)+1):]=='.'+str(suffix):
                    zd={"type":"file","path":folder+"/"+files}
                    lists.append(zd)
    return lists


def index(): #浏览器打开该地址：http://0.0.0.0:39001/index/index
    return returnjson({"desc":"您好，这是基与kcw开发的api框架，注意：您应该是在app/web/view/目录下开发接口，接下来祝您的开发愉快，","name":"/index/index"})
def test1(): #浏览器打开该地址：http://0.0.0.0:39001/index/test1
    return returnjson({"desc":"没错，您看到的就只有这点json字符串","name":"/index/test1"})
def html():
    return render_template("index/html.html",title="标题",desc="描述",content="这就是html模板")


from config import *
app['app_name']=''                                  #应用名称
app['app_debug']=True                               #是否开启调试模式
app['SECRET_KEY']='reresgrdsafagrewgregrtdge'       #session密钥
app['template_folder']='temp'                       ##设置模板文件目录名 注意：不能配置目录路径 

#签权/登录配置
oauth['status']=False    #签权开关
oauth['need']=[	        # “需要” 签权的视图函数
	"*" 	        # *：表示全部。 /index*:表示index视图文件下的所有方法。  /v1*：表示v1下的所有。/v1/index*：表示v1/index下的所有
]
oauth['unwanted']=[		# “不需要” 签权的视图函数
	"login/*",	        #表示login下所有不需要签权
]

route['defaultparam']=0    #设置路由中的参数名在url上不需要显示
route['/index/index']=[{'route':"/<html>","methods":["GET"]},{'route':"/","methods":["GET"]}]





from config import *
# 默认数据库配置
database['pattern']=True # True数据库长连接模式 False数据库短连接模式  注：建议web应用使用短连接，cli应用使用长连接
database['cli']=True # 是否以cli方式运行
database['dbObjcount']=1 # 连接池数量（单个数据库地址链接数量），数据库链接实例数量 mysql长链接模式下有效







# python setup.py bdist_egg     # 生成类似 edssdk-0.0.1-py2.7.egg，支持 easy_install 
# python setup.py sdist         # 生成类似 edssdk-0.0.1.tar.gz，支持 pip
# 打包后的dist上传到PyPi服务器
# python setup.py sdist upload
#############################################
from setuptools import setup, find_packages,Extension
from kcw.config import confkcw
import os
def get_file(folder='./',lists=[]):
    lis=os.listdir(folder)
    for files in lis:
        if not os.path.isfile(folder+"/"+files):
            if files!='__pycache__':
                lists.append(folder+"/"+files)
            get_file(folder+"/"+files,lists)
        else:
            pass
    return lists
b=get_file("kcw",['kcw'])
setup(
    name = confkcw["name"],
    version = confkcw["version"], #版本号
    description = confkcw["description"],
    author_email = confkcw["author_email"],
    packages =  b,
    install_requires = confkcw["install_requires"] #这是一个数组，里边包含的是咱的pip项目引用到的第三方库，如果你的项目有用到第三方库，要在这里添上第三方库的包名，如果用的第三方版本不是最新版本，还要有版本号。
)


# -*- coding: utf-8 -*-
import os,sys,re,getopt,time
sys.path.append("../../")
ar=sys.argv
if len(ar) <=1:
    exit("请查阅修改cli文档来输入正确的命令参数")
try:
    opts, args = getopt.getopt(sys.argv[1:], 'df:n:s:', ['help', 'input='])
except Exception as e:
    exit("请查阅修改cli文档来输入正确的命令参数")
f=''  #要执行的文件 test/aa,test/bb分别表示start/test/aa.py和start/test/bb.py
d=False
n=1
s='start'  #start启动  stop停止 restart重启
for v in opts:
    # print(v[0])
    if v[0] == '-d':  #守护进程运行
        d=True
    if v[0] == '-f':  #要执行的文件
        f=v[1]
    if v[0] == '-n':  #运行数量
        n=v[1]
    if v[0] == '-s':  #运行数量
        s=v[1]
n=int(n)
if 'linux' in sys.platform:
    if s=='start':
        pass
    elif s=='stop':
        os.system(r"pkill python3flask_pl")
        exit()
    elif s=='restart':
        os.system(r"pkill python3flask_pl")
    else:
        exit("-s参数超出范围")
elif sys.platform == 'win32' or sys.platform == 'win64':
    if s=='start':
        pass
    elif s=='stop':
        os.system(r"taskkill  /f  /im  python3flask_plus.exe")
        exit()
    elif s=='restart':
        os.system(r"taskkill  /f  /im  python3flask_plus.exe")
    else:
        exit("-s参数超出范围")
else:
    exit("不支持此操作系统",sys.platform)

f=f.split(',')
zxlist=[]
for k in f:
    # print("start/"+str(k)+".py")
    if os.path.isfile("start/"+str(k)+".py"):
        result = k.split('/')
        kk={'path':"start/"+re.sub(result[len(result)-1],'',k),'file':result[len(result)-1]+'.py'}
        zxlist.append(kk)
    else:
        exit("-f参数错误，找不到相关文件")



        
if d:
    if 'linux' in sys.platform:
        for k in zxlist:
            i=0
            while i< n:
                time.sleep(0.1)
                os.system(r"cd "+k['path']+"&& python3flask_plus "+k['file']+" >>./log &")
                i=i+1
    elif sys.platform == 'win32' or sys.platform == 'win64':
        for k in zxlist:
            i=0
            while i< n:
                time.sleep(0.1)
                os.system(r"cd "+k['path']+"&& start /b python3flask_plus "+k['file'])
                i=i+1
    else:
        exit("此操作系统不支持-d方式运行")
else:
    for k in zxlist:
        i=0
        while i< n:
            time.sleep(0.1)
            os.system(r"cd "+k['path']+"&& python3flask_plus "+k['file'])
            i=i+1
